package com.inhand.inhand_scan

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import org.json.JSONObject
import java.util.*

/** InhandScanPlugin */
class InhandScanPlugin: FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private lateinit var eventChannel : EventChannel
  private var applicationContext: Context? = null

  /// 广播名 ACTION
  private val ACTION_DATA_CODE_RECEIVED = "nlscan.action.SCANNER_RESULT"

  // flutter app里面要监听数据流,先不管
  private val CHARGING_CHANNEL = "inhand_scan/send"

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "inhand_scan")
    channel.setMethodCallHandler(this)
    eventChannel = EventChannel(flutterPluginBinding.getBinaryMessenger(), CHARGING_CHANNEL)
    eventChannel.setStreamHandler(object : StreamHandler() {
      private var chargingStateChangeReceiver: BroadcastReceiver? = null
      fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        chargingStateChangeReceiver = createChargingStateChangeReceiver(events)
        val filter = IntentFilter()
        filter.addAction(ACTION_DATA_CODE_RECEIVED)
        applicationContext.registerReceiver(
                chargingStateChangeReceiver, filter)
      }

      fun onCancel(arguments: Any?) {
        applicationContext.unregisterReceiver(chargingStateChangeReceiver)
        chargingStateChangeReceiver = null
      }
    })

    applicationContext = flutterPluginBinding.getApplicationContext()
  }

  fun registerWith(registrar: Registrar) {
    val channel = MethodChannel(registrar.messenger(), "inhand_scan")
    channel.setMethodCallHandler(InhandScanPlugin())
  }
  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "getPlatformVersion") {
      result.success("Android ${android.os.Build.VERSION.RELEASE}")
    } else {
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }
  private fun createChargingStateChangeReceiver(events: EventChannel.EventSink): BroadcastReceiver? {
    return object : BroadcastReceiver() {
      override fun onReceive(context: Context, intent: Intent) {
        val barocode = intent.getStringExtra("SCAN_BARCODE1")
        val resultMap: MutableMap<String?, Any?> = HashMap()
        if (barocode != null) {
          //成功
          resultMap["result"] = barocode

          // Map 转 json
          val json = JSONObject(resultMap)
          events.success(json.toString())
        } else {
          //成功
          resultMap["error"] = "请求超时，请重新扫描"

          // Map 转 json
          val json = JSONObject(resultMap)
          events.success(json.toString())
        }
      }
    }
  }
}
